@@ -1,7 +1,6 @@
-require_relative 'helper'
+require_relative "helper"
 
 class TestConnectionPool < Minitest::Test
-
   class NetworkConnection
     SLEEP_TIME = 0.1
 
@@ -9,8 +8,14 @@ class TestConnectionPool < Minitest::Test
       @x = 0
     end
 
-    def do_something
-      @x += 1
+    def do_something(*_args, increment: 1)
+      @x += increment
+      sleep SLEEP_TIME
+      @x
+    end
+
+    def do_something_with_positional_hash(options)
+      @x += options[:increment] || 1
       sleep SLEEP_TIME
       @x
     end
@@ -43,12 +48,12 @@ class TestConnectionPool < Minitest::Test
   end
 
   def use_pool(pool, size)
-    Array.new(size) do
+    Array.new(size) {
       Thread.new do
-        pool.with do sleep end
+        pool.with { sleep }
       end
-    end.each do |thread|
-      Thread.pass until thread.status == 'sleep'
+    }.each do |thread|
+      Thread.pass until thread.status == "sleep"
     end
   end
 
@@ -67,13 +72,13 @@ class TestConnectionPool < Minitest::Test
 
     generations = 3
 
-    result = Array.new(pool_size * generations) do
+    result = Array.new(pool_size * generations) {
       Thread.new do
         pool.with do |net|
           net.do_something
         end
       end
-    end.map(&:value)
+    }.map(&:value)
 
     finish = Time.new
 
@@ -84,14 +89,14 @@ class TestConnectionPool < Minitest::Test
 
   def test_timeout
     pool = ConnectionPool.new(timeout: 0, size: 1) { NetworkConnection.new }
-    thread = Thread.new do
+    thread = Thread.new {
       pool.with do |net|
         net.do_something
         sleep 0.01
       end
-    end
+    }
 
-    Thread.pass while thread.status == 'run'
+    Thread.pass while thread.status == "run"
 
     assert_raises Timeout::Error do
       pool.with { |net| net.do_something }
@@ -108,31 +113,46 @@ class TestConnectionPool < Minitest::Test
     pool = ConnectionPool.new(timeout: 0, size: 1) { Object.new }
 
     pool.with do
-      assert_raises Timeout::Error do
-        Thread.new { pool.checkout }.join
-      end
+      Thread.new {
+        assert_raises Timeout::Error do
+          pool.checkout
+        end
+      }.join
     end
 
     assert Thread.new { pool.checkout }.join
   end
 
+  def test_then
+    pool = ConnectionPool.new { Object.new }
+
+    assert_equal pool.method(:then), pool.method(:with)
+  end
+
   def test_with_timeout
     pool = ConnectionPool.new(timeout: 0, size: 1) { Object.new }
 
     assert_raises Timeout::Error do
       Timeout.timeout(0.01) do
         pool.with do |obj|
-          assert_equal 0, pool.instance_variable_get(:@available).instance_variable_get(:@que).size
+          assert_equal 0, pool.available
           sleep 0.015
         end
       end
     end
-    assert_equal 1, pool.instance_variable_get(:@available).instance_variable_get(:@que).size
+    assert_equal 1, pool.available
   end
 
-  def test_checkout_ignores_timeout
-    skip("Thread.handle_interrupt not available") unless Thread.respond_to?(:handle_interrupt)
+  def test_invalid_size
+    assert_raises ArgumentError, TypeError do
+      ConnectionPool.new(timeout: 0, size: nil) { Object.new }
+    end
+    assert_raises ArgumentError, TypeError do
+      ConnectionPool.new(timeout: 0, size: "") { Object.new }
+    end
+  end
 
+  def test_handle_interrupt_ensures_checkin
     pool = ConnectionPool.new(timeout: 0, size: 1) { Object.new }
     def pool.checkout(options)
       sleep 0.015
@@ -140,7 +160,8 @@ class TestConnectionPool < Minitest::Test
     end
 
     did_something = false
-    assert_raises Timeout::Error do
+
+    action = lambda do
       Timeout.timeout(0.01) do
         pool.with do |obj|
           did_something = true
@@ -152,18 +173,33 @@ class TestConnectionPool < Minitest::Test
         end
       end
     end
-    assert did_something
-    assert_equal 1, pool.instance_variable_get(:@available).instance_variable_get(:@que).size
+
+    if RUBY_ENGINE == "ruby"
+      # These asserts rely on the Ruby implementation reaching `did_something =
+      # true` before the interrupt is detected by the thread. Interrupt
+      # detection timing is implementation-specific in practice, with JRuby,
+      # Rubinius, and TruffleRuby all having different interrupt timings to MRI.
+      # In fact they generally detect interrupts more quickly than MRI, so they
+      # may not reach `did_something = true` before detecting the interrupt.
+
+      assert_raises Timeout::Error, &action
+
+      assert did_something
+    else
+      action.call
+    end
+
+    assert_equal 1, pool.available
   end
 
   def test_explicit_return
-    pool = ConnectionPool.new(timeout: 0, size: 1) do
+    pool = ConnectionPool.new(timeout: 0, size: 1) {
       mock = Minitest::Mock.new
       def mock.disconnect!
         raise "should not disconnect upon explicit return"
       end
       mock
-    end
+    }
 
     pool.with do |conn|
       return true
@@ -173,14 +209,14 @@ class TestConnectionPool < Minitest::Test
   def test_with_timeout_override
     pool = ConnectionPool.new(timeout: 0, size: 1) { NetworkConnection.new }
 
-    t = Thread.new do
+    t = Thread.new {
       pool.with do |net|
         net.do_something
         sleep 0.01
       end
-    end
+    }
 
-    Thread.pass while t.status == 'run'
+    Thread.pass while t.status == "run"
 
     assert_raises Timeout::Error do
       pool.with { |net| net.do_something }
@@ -195,9 +231,11 @@ class TestConnectionPool < Minitest::Test
     pool = ConnectionPool.new(timeout: 0, size: 1) { NetworkConnection.new }
     conn = pool.checkout
 
-    assert_raises Timeout::Error do
-      Thread.new { pool.checkout }.join
-    end
+    Thread.new {
+      assert_raises Timeout::Error do
+        pool.checkout
+      end
+    }.join
 
     pool.checkin
 
@@ -206,17 +244,14 @@ class TestConnectionPool < Minitest::Test
 
   def test_returns_value
     pool = ConnectionPool.new(timeout: 0, size: 1) { Object.new }
-    assert_equal 1, pool.with {|o| 1 }
+    assert_equal 1, pool.with { |o| 1 }
   end
 
   def test_checkin_never_checkout
     pool = ConnectionPool.new(timeout: 0, size: 1) { Object.new }
 
-    e = assert_raises ConnectionPool::Error do
-      pool.checkin
-    end
-
-    assert_equal 'no connections are checked out', e.message
+    e = assert_raises(ConnectionPool::Error) { pool.checkin }
+    assert_equal "no connections are checked out", e.message
   end
 
   def test_checkin_no_current_checkout
@@ -238,11 +273,11 @@ class TestConnectionPool < Minitest::Test
 
     pool.checkin
 
-    assert_raises Timeout::Error do
-      Thread.new do
+    Thread.new {
+      assert_raises Timeout::Error do
         pool.checkout
-      end.join
-    end
+      end
+    }.join
 
     pool.checkin
 
@@ -263,9 +298,9 @@ class TestConnectionPool < Minitest::Test
     pool = ConnectionPool.new(size: 2) { NetworkConnection.new }
     conn = pool.checkout
 
-    t = Thread.new do
+    t = Thread.new {
       pool.checkout
-    end
+    }
 
     refute_same conn, t.value
   end
@@ -281,14 +316,14 @@ class TestConnectionPool < Minitest::Test
   def test_checkout_timeout_override
     pool = ConnectionPool.new(timeout: 0, size: 1) { NetworkConnection.new }
 
-    thread = Thread.new do
+    thread = Thread.new {
       pool.with do |net|
         net.do_something
         sleep 0.01
       end
-    end
+    }
 
-    Thread.pass while thread.status == 'run'
+    Thread.pass while thread.status == "run"
 
     assert_raises Timeout::Error do
       pool.checkout
@@ -303,6 +338,8 @@ class TestConnectionPool < Minitest::Test
     assert_equal 2, pool.do_something
     assert_equal 5, pool.do_something_with_block { 3 }
     assert_equal 6, pool.with { |net| net.fast }
+    assert_equal 8, pool.do_something(increment: 2)
+    assert_equal 10, pool.do_something_with_positional_hash({ increment: 2, symbol_key: 3, "string_key" => 4 })
   end
 
   def test_passthru_respond_to
@@ -315,22 +352,22 @@ class TestConnectionPool < Minitest::Test
 
   def test_return_value
     pool = ConnectionPool.new(timeout: 2 * NetworkConnection::SLEEP_TIME, size: 1) { NetworkConnection.new }
-    result = pool.with do |net|
+    result = pool.with { |net|
       net.fast
-    end
+    }
     assert_equal 1, result
   end
 
   def test_heavy_threading
     pool = ConnectionPool.new(timeout: 0.5, size: 3) { NetworkConnection.new }
 
-    threads = Array.new(20) do
+    threads = Array.new(20) {
       Thread.new do
         pool.with do |net|
           sleep 0.01
         end
       end
-    end
+    }
 
     threads.map { |thread| thread.join }
   end
@@ -338,9 +375,9 @@ class TestConnectionPool < Minitest::Test
   def test_reuses_objects_when_pool_not_saturated
     pool = ConnectionPool.new(size: 5) { NetworkConnection.new }
 
-    ids = 10.times.map do
+    ids = 10.times.map {
       pool.with { |c| c.object_id }
-    end
+    }
 
     assert_equal 1, ids.uniq.size
   end
@@ -349,32 +386,32 @@ class TestConnectionPool < Minitest::Test
     recorder = Recorder.new
     pool = ConnectionPool.new(size: 1) { recorder }
     pool.with do |r_outer|
-      @other = Thread.new do |t|
+      @other = Thread.new { |t|
         pool.with do |r_other|
-          r_other.do_work('other')
+          r_other.do_work("other")
         end
-      end
+      }
 
       pool.with do |r_inner|
-        r_inner.do_work('inner')
+        r_inner.do_work("inner")
       end
 
       Thread.pass
 
-      r_outer.do_work('outer')
+      r_outer.do_work("outer")
     end
 
     @other.join
 
-    assert_equal ['inner', 'outer', 'other'], recorder.calls
+    assert_equal ["inner", "outer", "other"], recorder.calls
   end
 
   def test_shutdown_is_executed_for_all_connections
     recorders = []
 
-    pool = ConnectionPool.new(size: 3) do
+    pool = ConnectionPool.new(size: 3) {
       Recorder.new.tap { |r| recorders << r }
-    end
+    }
 
     threads = use_pool pool, 3
 
@@ -390,7 +427,7 @@ class TestConnectionPool < Minitest::Test
   def test_raises_error_after_shutting_down
     pool = ConnectionPool.new(size: 1) { true }
 
-    pool.shutdown { }
+    pool.shutdown {}
 
     assert_raises ConnectionPool::PoolShuttingDownError do
       pool.checkout
@@ -400,9 +437,9 @@ class TestConnectionPool < Minitest::Test
   def test_runs_shutdown_block_asynchronously_if_connection_was_in_use
     recorders = []
 
-    pool = ConnectionPool.new(size: 3) do
+    pool = ConnectionPool.new(size: 3) {
       Recorder.new.tap { |r| recorders << r }
-    end
+    }
 
     threads = use_pool pool, 2
 
@@ -422,7 +459,7 @@ class TestConnectionPool < Minitest::Test
   end
 
   def test_raises_an_error_if_shutdown_is_called_without_a_block
-    pool = ConnectionPool.new(size: 1) { }
+    pool = ConnectionPool.new(size: 1) {}
 
     assert_raises ArgumentError do
       pool.shutdown
@@ -432,9 +469,9 @@ class TestConnectionPool < Minitest::Test
   def test_shutdown_is_executed_for_all_connections_in_wrapped_pool
     recorders = []
 
-    wrapper = ConnectionPool::Wrapper.new(size: 3) do
+    wrapper = ConnectionPool::Wrapper.new(size: 3) {
       Recorder.new.tap { |r| recorders << r }
-    end
+    }
 
     threads = use_pool wrapper, 3
 
@@ -447,6 +484,11 @@ class TestConnectionPool < Minitest::Test
     assert_equal [["shutdown"]] * 3, recorders.map { |r| r.calls }
   end
 
+  def test_wrapper_wrapped_pool
+    wrapper = ConnectionPool::Wrapper.new { NetworkConnection.new }
+    assert_equal ConnectionPool, wrapper.wrapped_pool.class
+  end
+
   def test_wrapper_method_missing
     wrapper = ConnectionPool::Wrapper.new { NetworkConnection.new }
 
@@ -466,14 +508,14 @@ class TestConnectionPool < Minitest::Test
     wrapper = ConnectionPool::Wrapper.new(timeout: 0, size: 1) { Object.new }
 
     wrapper.with do
-      assert_raises Timeout::Error do
-        Thread.new do
-          wrapper.with { flunk 'connection checked out :(' }
-        end.join
-      end
+      Thread.new {
+        assert_raises Timeout::Error do
+          wrapper.with { flunk "connection checked out :(" }
+        end
+      }.join
     end
 
-    assert Thread.new { wrapper.with { } }.join
+    assert Thread.new { wrapper.with {} }.join
   end
 
   class ConnWithEval
@@ -493,10 +535,10 @@ class TestConnectionPool < Minitest::Test
     pool = ConnectionPool.new(size: 1) { recorder }
     wrapper = ConnectionPool::Wrapper.new(pool: pool)
 
-    pool.with { |r| r.do_work('with') }
-    wrapper.do_work('wrapped')
+    pool.with { |r| r.do_work("with") }
+    wrapper.do_work("wrapped")
 
-    assert_equal ['with', 'wrapped'], recorder.calls
+    assert_equal ["with", "wrapped"], recorder.calls
   end
 
   def test_stats_without_active_connection
@@ -513,4 +555,13 @@ class TestConnectionPool < Minitest::Test
       assert_equal(1, pool.available)
     end
   end
+
+  def test_stats_with_string_size
+    pool = ConnectionPool.new(size: "2") { NetworkConnection.new }
+
+    pool.with do
+      assert_equal(2, pool.size)
+      assert_equal(1, pool.available)
+    end
+  end
 end
